'''Helper with policy templates'''
import re

class Policy: # pylint: disable=too-few-public-methods
  '''Policy class'''

  @staticmethod
  def merge(policies):
    '''Helper to merge policies'''
    all_statements = []
    for policy in policies:
      all_statements += policy.get('Statement')
    return {
      'Statement': all_statements,
      'Version': '2012-10-17',
    }

  @staticmethod
  def ecr_bio_modules(repository_settings):
    '''ECR policy for worker'''
    return {
      'Statement': [{
        'Effect': 'Allow',
        'Action': [
          'ecr:BatchCheckLayerAvailability',
          'ecr:GetDownloadUrlForLayer',
          'ecr:GetRepositoryPolicy',
          'ecr:DescribeRepositories',
          'ecr:ListImages',
          'ecr:DescribeImages',
          'ecr:BatchGetImage',
          'ecr:GetLifecyclePolicy',
          'ecr:GetLifecyclePolicyPreview',
          'ecr:ListTagsForResource',
          'ecr:DescribeImageScanFindings'
        ],
        'Resource': [
          f"arn:aws:ecr:*:{repository_settings.get('account')}:repository/bio-*",
          f"arn:aws:ecr:*:{repository_settings.get('account')}:repository/takara*",
          f"arn:aws:ecr:*:{repository_settings.get('account')}:repository/vitrolife*",
        ],
        'Condition': {
          'StringEquals': {
            'aws:ResourceTag/Type': 'Bio'
          }
        }
      }, {
        'Effect': 'Allow',
        'Action': 'ecr:GetAuthorizationToken',
        'Resource': '*'
      }, {
          'Effect': 'Allow',
          'Action': [
              'cloudwatch:PutMetricData',
              'ec2:DescribeVolumes',
              'ec2:DescribeTags',
              'logs:PutLogEvents',
              'logs:DescribeLogStreams',
              'logs:DescribeLogGroups',
              'logs:CreateLogStream',
              'logs:CreateLogGroup'
          ],
          'Resource': '*'
      }, {
          'Effect': 'Allow',
          'Action': ['ssm:GetParameter'],
          'Resource': 'arn:aws:ssm:*:*:parameter/AmazonCloudWatch-*'
      }],
      'Version': '2012-10-17'
    }

  @staticmethod
  def s3_delete_purpose(bucket, user_id):
    '''S3 policy to allow delete permission to their own uploaded samples'''
    resource = [
      f'arn:aws:s3:::{bucket}/uploads/{user_id}/*', # add sample_id to further restrict the user
    ]
    return {
      'Statement': [{
        'Action': ['s3:DeleteObject'],
        'Effect': 'Allow',
        'Resource': resource,
      }],
      'Version': '2012-10-17'
    }

  @staticmethod
  def hos_general_purpose(ho_region, reference_store_id, sequence_store_id):
    '''HOS policy template for general purpose'''
    return {
      'Statement': [
        {
          'Action': [
            "omics:BatchDeleteReadSet",
            "omics:DeleteReference",
            "omics:GetReadSet",
            "omics:GetReadSetExportJob",
            "omics:GetReadSetImportJob",
            "omics:GetReadSetMetadata",
            "omics:GetReferenceImportJob",
            "omics:GetReferenceMetadata",
            "omics:ListReadSets",
            "omics:ListReferences",
            "omics:StartReadSetActivationJob",
            "omics:StartReadSetExportJob",
            "omics:StartReadSetImportJob",
            "omics:StartReferenceImportJob",
          ],
          'Effect': 'Allow',
          'Resource': [
            f"arn:aws:omics:{ho_region}:*:referenceStore/{reference_store_id}/*",
            f"arn:aws:omics:{ho_region}:*:referenceStore/{reference_store_id}",
            f"arn:aws:omics:{ho_region}:*:sequenceStore/{sequence_store_id}/*",
            f"arn:aws:omics:{ho_region}:*:sequenceStore/{sequence_store_id}",
          ]
        }, {
          'Action': [
            "s3:GetObject",
            "s3:ListBucket"
          ],
          'Effect': 'Allow',
          'Resource': ['*'],
          'Condition': {
            'StringLike': {
              "s3:DataAccessPointArn": f"arn:aws:s3:{ho_region}:*"
            }
          }
        }, {
          'Action': ["kms:Decrypt"],
          'Effect': 'Allow',
          'Resource': [f"arn:aws:kms:{ho_region}:*:*"]
        }
      ]
    }

  @staticmethod
  def s3_general_purpose(bucket, reflib_buckets, user_id):
    '''S3 policy template for general purpose'''
    resource = [
      f'arn:aws:s3:::{bucket}/analyses/{user_id}/*',
      f'arn:aws:s3:::{bucket}/log/analyses/{user_id}/*',
      f'arn:aws:s3:::{bucket}/uploads/{user_id}/*',
    ]

    if 'vitrolife' in bucket:
      resource.append(f'arn:aws:s3:::{bucket}/vitrolife/*')
    if bool(re.search(r'(ivf|takara)', bucket)):
      resource.append(f'arn:aws:s3:::{bucket}/ivf/*')

    return {
      'Statement': [{
        'Action': 's3:ListBucket',
        'Effect': 'Allow',
        'Resource': [f'arn:aws:s3:::{bucket}'],
      }, {
        'Action': ['s3:GetObject', 's3:PutObject', 's3:PutObjectAcl', 's3:PutObjectTagging'],
        'Effect': 'Allow',
        'Resource': resource,
      }, {
        'Action': ['s3:GetObject'],
        'Effect': 'Allow',
        'Resource': [
          f'arn:aws:s3:::{reflib_bucket}/reflib/*' for reflib_bucket in reflib_buckets
        ],
      }, {
        'Action': 's3:ListBucket',
        'Effect': 'Allow',
        'Condition': {
          'StringLike': {
            's3:prefix': 'reflib/*'
          }
        },
        'Resource': [
          f'arn:aws:s3:::{reflib_bucket}' for reflib_bucket in reflib_buckets
        ],
      }],
      'Version': '2012-10-17'
    }

  @staticmethod
  def s3_upload(bucket, user_id):
    '''S3 Policy template for uploading purpose'''
    return {
      'Statement': [{
        'Effect': 'Allow',
        'Action': [
          's3:AbortMultipartUpload',
          's3:ListMultipartUploadParts',
          's3:PutObject',
        ],
        'Resource': [f'arn:aws:s3:::{bucket}/uploads/{user_id}/*']
      }],
      'Version': '2012-10-17'
    }

  @staticmethod
  def swf_worker(workflow_settings):
    '''SWF policy for worker'''
    return {
      'Statement': [{
        'Action': [
          'swf:RespondActivityTaskCompleted',
          'swf:PollForActivityTask',
        ],
        'Effect': 'Allow',
        'Resource': f"arn:aws:swf:*:*:/domain/{workflow_settings.get('domain')}"
      }],
      'Version': '2012-10-17'
    }


'''AWS HealthOmics Workflow Wrapper'''

# Libs imports
from botocore.client import Config
from botocore.exceptions import ClientError

# Module imports
from basepair.modules.aws.service import Service

class HOW(Service):
  '''Wrapper for Health Omics Workflow services'''

  def __init__(self, cfg):
    super().__init__(cfg, 'HOW')
    self.client = self.session.client(**{
      'config': Config(retries={'max_attempts': 0, 'mode': 'standard'}),
      'service_name': 'omics',
    })

  def create_workflow(self, params):
    '''Create workflow'''
    try:
      response = self.client.create_workflow(**params)
    except ClientError as error:
      self.get_log_msg({
        'exception': error,
        'msg': f'Not able to create HealthOmics workflow: {str(error)}.',
      })
      raise error
    return response

  def cancel_run(self, run_id):
    '''Cancel run'''
    try:
      response = self.client.cancel_run(id=run_id)
    except ClientError as error:
      self.get_log_msg({
        'exception': error,
        'msg': f'Not able to cancel HealthOmics run: {str(error)}.',
      })
      raise error
    return response

  def delete_run(self, run_id):
    '''Delete run'''
    try:
      response = self.client.delete_run(id=run_id)
    except ClientError as error:
      self.get_log_msg({
        'exception': error,
        'msg': f'Not able to delete HealthOmics run: {str(error)}.',
      })
      raise error
    return response

  def delete_workflow(self, params):
    '''Delete workflow'''
    try:
      response = self.client.delete_workflow(**params)
    except ClientError as error:
      self.get_log_msg({
        'exception': error,
        'msg': f'Not able to delete HealthOmics workflow: {str(error)}.',
      })
      raise error
    return response

  def get_run(self, params):
    '''Get omics run'''
    try:
      return self.client.get_run(**params)
    except ClientError as error:
      self.get_log_msg({
        'exception': str(error),
        'msg': f'Not able to Get HealthOmics run id {params.get("id")}: {str(error)}.',
      })
      raise error
  
  def get_run_task(self, params):
    '''Get omics run task'''
    try:
      return self.client.get_run_task(**params)
    except ClientError as error:
      self.get_log_msg({
        'exception': str(error),
        'msg': f'Not able to Get HealthOmics run task id {params.get("id")}: {str(error)}.',
      })
      raise error

  def get_workflow(self, params):
    '''Get omics workflow'''
    try:
      return self.client.get_workflow(**params)
    except ClientError as error:
      self.get_log_msg({
        'exception': str(error),
        'msg': f'Not able to get HealthOmics workflow: {str(error)}.',
      })
      raise error

  def list_run_tasks(self, params):
    '''Get omics run list'''
    try:
      tasks = []
      while True:
        response = self.client.list_run_tasks(**params)
        next_token = response.get("nextToken")
        tasks += response.get('items')
        if next_token:
          params["startingToken"] = next_token
        else:
          break
      return tasks
    except ClientError as error:
      self.get_log_msg({
        'exception':error,
        'msg': f'Not able to Get HealthOmics run task list id {params.get("id")}: {str(error)}'
      })
      raise error

  def list_workflows(self, params):
    '''List omics workflows'''
    try:
      return self.client.list_workflows(**params)
    except ClientError as error:
      self.get_log_msg({
        'exception': str(error),
        'msg': f'Not able to list HealthOmics workflows: {str(error)}.',
      })
      raise error

  def start_run(self, params):
    '''Start omics workflow'''
    try:
      return self.client.start_run(**params)
    except ClientError as error:
      self.get_log_msg({
        'exception': str(error),
        'msg': f'Not able to start HealthOmics workflow id {params.get("workflowId")}: {str(error)}.',
      })
      raise error


'''AWS SM Wrappers'''

# General import
import json
import os
import time

# Libs imports
from botocore.client import Config
from botocore.exceptions import ClientError

# Module imports
from basepair.modules.aws.handler.exception import ExceptionHandler
from basepair.modules.aws.service import Service

class SM(Service): # pylint: disable=too-few-public-methods
  '''Wrapper for CW services'''
  def __init__(self, cfg):
    super().__init__(cfg, 'SM')
    self.client = self.session.client(**{
      'config': Config(retries={'max_attempts': 0, 'mode': 'standard'}),
      'service_name': 'secretsmanager',
    })

  def get(self, secret_id, use_cache=True):
    '''Get the values for the given secret key'''
    key = f'{secret_id}'
    cache_file = f'/tmp/{key}'
    twelve_hours_ago = time.time() - 43200 # 43200 seconds = 12hs
    if use_cache and os.path.isfile(cache_file):
      # reading the price from cache
      cache = {}
      with open(cache_file, 'r', encoding='utf-8') as file:
        cache = json.load(file)
      # expire the cache if needed
      filte_stats = os.stat(cache_file)
      if filte_stats.st_mtime < twelve_hours_ago:
        os.unlink(cache_file)
      # returning cached price
      return cache.get('SecretString', {})

    try:
      response = self.client.get_secret_value(SecretId=secret_id)
    except ClientError as error:
      response = self.get_log_msg({
        'exception': error,
        'msg': f'Not able to get secret with id {secret_id}.',
      })
      if ExceptionHandler.is_throttled_error(exception=error):
        raise error
      return response
    secret = response.get('SecretString')
    if secret and use_cache:
      with open(cache_file, 'w', encoding='utf-8') as file:
        json_content = json.dumps({'SecretString': secret})
        file.write(json_content)
    return secret
